"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WrappedError = exports.CustomError = exports.tryGetJsonError = exports.convertErrorToPlainObject = exports.getJsonError = exports.getErrorFields = exports.fixErrorPrototype = void 0;
var tslib_1 = require("tslib");
var ts_custom_error_1 = require("ts-custom-error");
/**
 * Исправляет прототип инстанса класса, расширяющего Error
 *
 * copy-pasted from https://github.com/adriengibrat/ts-custom-error/blob/master/src/utils.ts
 *
 * @param error - инстанс класса
 * @param prototype - целевой прототип
 *
 * @see https://stackoverflow.com/questions/41102060/typescript-extending-error-class
 */
function fixErrorPrototype(error, prototype) {
    if (prototype === void 0) { prototype = error.constructor.prototype; }
    var setPrototypeOf = Object.setPrototypeOf;
    if (setPrototypeOf) {
        setPrototypeOf(error, prototype);
    }
    else {
        error.__proto__ = prototype;
    }
    error.name = prototype.name;
}
exports.fixErrorPrototype = fixErrorPrototype;
/**
 * Возвращает представление ошибки в виде простого объекта.
 * Полезно для сериализации в JSON
 *
 * @param error - объект ошибки
 */
function getErrorFields(error) {
    var result = {};
    for (var _i = 0, _a = Object.getOwnPropertyNames(error); _i < _a.length; _i++) {
        var prop = _a[_i];
        if (prop !== 'stack') {
            result[prop] = error[prop];
        }
    }
    return result;
}
exports.getErrorFields = getErrorFields;
/**
 * Алиас для getErrorFields
 * @deprecated используйте getErrorFields
 */
exports.getJsonError = getErrorFields;
/**
 * Возвращает представление ошибки в виде простого объекта.
 * Полезно для сериализации в JSON.
 * Пытается вызвать error.toJSON()
 *
 * @param error - объект ошибки
 */
function convertErrorToPlainObject(error) {
    if (error.toJSON && typeof error.toJSON === 'function') {
        return error.toJSON();
    }
    return getErrorFields(error);
}
exports.convertErrorToPlainObject = convertErrorToPlainObject;
/**
 * Алиас для convertErrorToPlainObject
 * @deprecated используйте convertErrorToPlainObject
 */
exports.tryGetJsonError = convertErrorToPlainObject;
/**
 * Базовый класс для ошибок уровня приложения.
 * Расширяйте его, чтобы избежать проблем с прототипом Error.
 *
 * Создавайте классы ошибок для того, чтобы различать разные типы ошибок
 * в коде с помощью instanceof.
 *
 * @see https://stackoverflow.com/questions/41102060/typescript-extending-error-class
 * @see https://github.com/adriengibrat/ts-custom-error
 *
 * @example
 * class UnexpectedResultError extends CustomError {
 *   constructor(public readonly expected: any, public readonly got: any) {
 *     super(`UnexpectedResultError: expected "${JSON.stringify(expected)}, got "${JSON.stringify(got)}"`);
 *   }
 * }
 *
 * fetch('/data')
 *   .then((result) => result.json())
 *   .then((data) => {
 *     if (!data.success) {
 *       throw new UnexpectedExternalApiResultError(true, data.success);
 *     }
 *   })
 *   .catch((error) => {
 *     if (e instanceof UnexpectedResultError) {
 *       toast.error('Неожиданный ответ сервера');
 *     }
 *     console.error(error);
 *   });
 *
 * @example
 * class UnexpectedExternalApiResultError extends UnexpectedResultError {}
 *
 * fetch('/data')
 *   .then((result) => result.json())
 *   .then((data) => {
 *     if (!data.success) {
 *       throw new UnexpectedExternalApiResultError(true, data.success);
 *     }
 *   })
 *   .catch((error) => {
 *     if (e instanceof UnexpectedResultError) {
 *       toast.error('Неожиданный ответ внешнего API');
 *     }
 *     console.error(error);
 *   });
 *
 */
var CustomError = /** @class */ (function (_super) {
    tslib_1.__extends(CustomError, _super);
    function CustomError(message) {
        var _this = _super.call(this, message) || this;
        // Fixes TypeError when redefining name in derived classes
        Object.defineProperty(_this, 'name', {
            value: 'CustomError',
            writable: true
        });
        return _this;
    }
    /**
     * Вызывается автоматически при использовании JSON.stringify
     */
    CustomError.prototype.toJSON = function () {
        return getErrorFields(this);
    };
    return CustomError;
}(ts_custom_error_1.CustomError));
exports.CustomError = CustomError;
/**
 * Базовый класс для ошибок, оборачивающих другую ошибку
 *
 * @example
 * class ForeignKeyError extends WrapperError {
 *   constructor(public readonly field: string, originalError: Error) {
 *     super('ForeignKeyError: ' + field, originalError);
 *   }
 * }
 *
 * function executeDbStatement() {
 *   try {
 *     // ...
 *   } catch (e) {
 *     if (e.message === 'foreign key error') {
 *       throw new ForeignKeyError(e.field, e);
 *     }
 *     throw e;
 *   }
 * }
 *
 * try {
 *   executeDbStatement();
 * } catch (e) {
 *   if (e instanceof ForeignKeyError) {
 *     console.log(e.message, e.field, e.stack, e.originalError.stack);
 *     // ...
 *   } else {
 *     // ...
 *   }
 * }
 */
var WrappedError = /** @class */ (function (_super) {
    tslib_1.__extends(WrappedError, _super);
    function WrappedError(message, originalError) {
        var _this = _super.call(this, message) || this;
        _this.originalError = originalError;
        return _this;
    }
    return WrappedError;
}(CustomError));
exports.WrappedError = WrappedError;
//# sourceMappingURL=error.js.map