import { __awaiter, __generator, __spreadArrays } from "tslib";
import { asyncify, eachOfLimit as eachOfLimitAsync, mapLimit as mapLimitAsync, } from 'async';
import chunk from 'lodash/chunk';
import concat from 'lodash/concat';
/**
 * Для каждого элемента array параллельно вызывает асинхронную функцию cb,
 * но не более limit одновременных вызовов.
 *
 * @param array - массив
 * @param limit - количество одновременно выполняемых операций cb
 * @param cb - колбек, принимающий элемент массива и его индекс
 */
export function eachOfLimit(array, limit, cb) {
    return eachOfLimitAsync(array, limit, asyncify(cb));
}
/**
 * Для каждого элемента array параллельно вызывает асинхронную функцию cb,
 * но не более limit одновременных вызовов.
 *
 * @param array - массив
 * @param limit - количество одновременно выполняемых операций cb
 * @param cb - колбек, принимающий элемент массива и его индекс и возвращающий результат
 * @returns result - массив результатов вызовов cb
 */
export function mapLimit(array, limit, cb) {
    return mapLimitAsync(array, limit, asyncify(cb));
}
/**
 * Для каждого элемента array параллельно вызывает асинхронную функцию cb,
 * но не более limit одновременных вызовов.
 *
 * Предполагается, что функция cb возвращает массив.
 * Конкатенирует результирующие массивы в один большой.
 *
 * @param array - массив
 * @param limit - количество одновременно выполняемых операций cb
 * @param cb - колбек, принимающий элемент массива и его индекс и возвращающий массив
 * @returns result - конкатенация результатов вызовов cb для каждого элемента array
 */
export function concatLimit(array, limit, cb) {
    return __awaiter(this, void 0, void 0, function () {
        var items;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, mapLimit(array, limit, cb)];
                case 1:
                    items = _a.sent();
                    return [2 /*return*/, concat.apply(void 0, __spreadArrays([[]], items))];
            }
        });
    });
}
/**
 * Разбивает массив array на чанки размера chunkSize и выполняет для каждого чанка
 * асинхронную функцию cb параллельно с ограничением в callLimit одновременных вызовов.
 *
 * @param array
 * @param chunkSize
 * @param callLimit
 * @param cb
 */
export function awaitChunkedPromises(array, chunkSize, callLimit, cb) {
    return __awaiter(this, void 0, void 0, function () {
        var chunks, nChunks;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    chunks = chunk(array, chunkSize);
                    nChunks = chunks.length;
                    return [4 /*yield*/, mapLimit(chunks, callLimit, function (slice, iSlice) {
                            return cb(slice, iSlice, nChunks);
                        })];
                case 1: return [2 /*return*/, _a.sent()];
            }
        });
    });
}
//# sourceMappingURL=async.js.map