import { CustomError as TsCustomError } from 'ts-custom-error';
/**
 * Исправляет прототип инстанса класса, расширяющего Error
 *
 * copy-pasted from https://github.com/adriengibrat/ts-custom-error/blob/master/src/utils.ts
 *
 * @param error - инстанс класса
 * @param prototype - целевой прототип
 *
 * @see https://stackoverflow.com/questions/41102060/typescript-extending-error-class
 */
export declare function fixErrorPrototype(error: any, prototype?: any): void;
/**
 * Возвращает представление ошибки в виде простого объекта.
 * Полезно для сериализации в JSON
 *
 * @param error - объект ошибки
 */
export declare function getErrorFields(error: any): Record<string, unknown>;
/**
 * Алиас для getErrorFields
 * @deprecated используйте getErrorFields
 */
export declare const getJsonError: typeof getErrorFields;
/**
 * Возвращает представление ошибки в виде простого объекта.
 * Полезно для сериализации в JSON.
 * Пытается вызвать error.toJSON()
 *
 * @param error - объект ошибки
 */
export declare function convertErrorToPlainObject(error: any): any;
/**
 * Алиас для convertErrorToPlainObject
 * @deprecated используйте convertErrorToPlainObject
 */
export declare const tryGetJsonError: typeof convertErrorToPlainObject;
/**
 * Базовый класс для ошибок уровня приложения.
 * Расширяйте его, чтобы избежать проблем с прототипом Error.
 *
 * Создавайте классы ошибок для того, чтобы различать разные типы ошибок
 * в коде с помощью instanceof.
 *
 * @see https://stackoverflow.com/questions/41102060/typescript-extending-error-class
 * @see https://github.com/adriengibrat/ts-custom-error
 *
 * @example
 * class UnexpectedResultError extends CustomError {
 *   constructor(public readonly expected: any, public readonly got: any) {
 *     super(`UnexpectedResultError: expected "${JSON.stringify(expected)}, got "${JSON.stringify(got)}"`);
 *   }
 * }
 *
 * fetch('/data')
 *   .then((result) => result.json())
 *   .then((data) => {
 *     if (!data.success) {
 *       throw new UnexpectedExternalApiResultError(true, data.success);
 *     }
 *   })
 *   .catch((error) => {
 *     if (e instanceof UnexpectedResultError) {
 *       toast.error('Неожиданный ответ сервера');
 *     }
 *     console.error(error);
 *   });
 *
 * @example
 * class UnexpectedExternalApiResultError extends UnexpectedResultError {}
 *
 * fetch('/data')
 *   .then((result) => result.json())
 *   .then((data) => {
 *     if (!data.success) {
 *       throw new UnexpectedExternalApiResultError(true, data.success);
 *     }
 *   })
 *   .catch((error) => {
 *     if (e instanceof UnexpectedResultError) {
 *       toast.error('Неожиданный ответ внешнего API');
 *     }
 *     console.error(error);
 *   });
 *
 */
export declare class CustomError extends TsCustomError {
    constructor(message?: string);
    /**
     * Вызывается автоматически при использовании JSON.stringify
     */
    toJSON(): Record<string, unknown>;
}
/**
 * Базовый класс для ошибок, оборачивающих другую ошибку
 *
 * @example
 * class ForeignKeyError extends WrapperError {
 *   constructor(public readonly field: string, originalError: Error) {
 *     super('ForeignKeyError: ' + field, originalError);
 *   }
 * }
 *
 * function executeDbStatement() {
 *   try {
 *     // ...
 *   } catch (e) {
 *     if (e.message === 'foreign key error') {
 *       throw new ForeignKeyError(e.field, e);
 *     }
 *     throw e;
 *   }
 * }
 *
 * try {
 *   executeDbStatement();
 * } catch (e) {
 *   if (e instanceof ForeignKeyError) {
 *     console.log(e.message, e.field, e.stack, e.originalError.stack);
 *     // ...
 *   } else {
 *     // ...
 *   }
 * }
 */
export declare class WrappedError extends CustomError {
    protected readonly originalError: Error;
    constructor(message: string, originalError: Error);
}
