/**
 * Класс, управляющий одним таймаутом.
 * Удобен в случаях, когда при назначении нового таймаута старый необходимо отменить,
 * либо когда таймаут должен быть отменен при наступлении внешнего события
 * независимо от того, установлен он или нет.
 *
 * По сути делает таймаут локальным, а не глобальным.
 *
 * Может использоваться с React-компонентами через хук useTimeoutRef.
 */
var SingleTimeoutManager = /** @class */ (function () {
    function SingleTimeoutManager() {
        /**
         * Идентификатор активного таймаута.
         * Равен null, если таймаут не запущен или уже сработал.
         */
        this.timeout = null;
    }
    /**
     * Запускает таймаут, отменяя предыдущий.
     *
     * @param cb - колбек, который будет вызван не раньше, чем через время time
     * @param time - время в мс, через которое будет вызван cb
     */
    SingleTimeoutManager.prototype.set = function (cb, time) {
        var _this = this;
        if (this.timeout)
            clearTimeout(this.timeout);
        this.timeout = setTimeout(function () {
            _this.timeout = null;
            cb === null || cb === void 0 ? void 0 : cb();
        }, time);
    };
    /**
     * Создает таймаут-пустышку.
     *
     * @param time - время в мс, через которое таймаут будет снят
     */
    SingleTimeoutManager.prototype.setDelay = function (time) {
        return this.set(null, time);
    };
    /**
     * Если таймаут запущен, то отменяет его.
     */
    SingleTimeoutManager.prototype.clear = function () {
        if (this.timeout) {
            clearTimeout(this.timeout);
            this.timeout = null;
        }
    };
    /**
     * Возвращает идентификатор активного таймаута
     */
    SingleTimeoutManager.prototype.active = function () {
        return !!this.timeout;
    };
    return SingleTimeoutManager;
}());
export { SingleTimeoutManager };
//# sourceMappingURL=SingleTimeoutManager.js.map