/**
 * Базовый компаратор. Сравнивает, используя операторы `<` и `>`
 * @param dir - направление сравнения (1 для asc, -1 для desc)
 */
export var compareSimple = function (dir) {
    if (dir === void 0) { dir = 1; }
    return function (a, b) {
        return a < b ? -dir : a > b ? dir : 0;
    };
};
/**
 * Создает функцию сравнения по полям объекта
 * @param compFn - компаратор
 * @param fieldFns - последовательность полей
 * @example
 *  const fn = makeComparator(compareSimple(), x => x.a, x => x.b)
 *  const sorted = [{a:2,b:1},{a:1,b:2},{a:1,b:1}].sort(fn);
 *  sorted -> [{a:1,b:1},{a:1,b:2},{a:2,b:1}]
 */
export function makeComparator(compFn) {
    var fieldFns = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        fieldFns[_i - 1] = arguments[_i];
    }
    return function (a, b) {
        for (var _i = 0, fieldFns_1 = fieldFns; _i < fieldFns_1.length; _i++) {
            var fieldFn = fieldFns_1[_i];
            var compRes = compFn(fieldFn(a), fieldFn(b));
            if (compRes !== 0)
                return compRes;
        }
        return 0;
    };
}
/**
 * Создает функцию сравнения по последовательности полей объекта.
 * По сравнению с makeComparator позволяет использовать разные функции
 * сравнения для разных полей
 *
 * @param steps - массив, элементами которого являются шаги сортировки
 *  - массивы из компараторов и подпоследовательности полей
 * @example
 *  const fn = makeMultiComparator([compareSimple(), x => x.a], [compareSimple(-1), x => x.b])
 *  const sorted = [{a:2,b:1},{a:1,b:2},{a:1,b:1}].sort(fn);
 *  sorted -> [{a:1,b:2},{a:1,b:1},{a:2,b:1}]
 */
export function makeMultiComparator() {
    var steps = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        steps[_i] = arguments[_i];
    }
    return function (a, b) {
        for (var _i = 0, steps_1 = steps; _i < steps_1.length; _i++) {
            var step = steps_1[_i];
            var compFn = step[0], fieldFns = step.slice(1);
            for (var _a = 0, fieldFns_2 = fieldFns; _a < fieldFns_2.length; _a++) {
                var fieldFn = fieldFns_2[_a];
                var compRes = compFn(fieldFn(a), fieldFn(b));
                if (compRes !== 0)
                    return compRes;
            }
        }
        return 0;
    };
}
//# sourceMappingURL=makeComparator.js.map