"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnumHelpers = void 0;
var tslib_1 = require("tslib");
/**
 * Набор утилит для работы с энумами (перечислениями).
 * Решает частые задачи, такие как формирование массивов опций для селектов,
 * поиск ключа по имени.
 *
 * @example
 * enum RoomType {
 *   Corridor = 'corridor',
 *   Class = 'class',
 *   Restroom = 'restroom',
 * }
 *
 * // Надо явно указать аргументы дженерика, потому что TS (v4.1) недостаточно умён
 * const roomType = new EnumHelpers<typeof RoomType, { name: string, order: number }>(
 *   RoomType,
 *   {
 *     [RoomType.Corridor]: { name: 'Коридор', order: 1 },
 *     [RoomType.Class]: { name: 'Аудитория', order: 2 },
 *     [RoomType.Restroom]: { name: 'Комната отдыха', order: 3 }
 *   }
 * );
 *
 * const roomTypes = roomType.getOptions().sort((a,b) => a.order - b.order);
 */
var EnumHelpers = /** @class */ (function () {
    /**
     * Конструктор
     * @param values - энум (enum), содержащее только ключи
     * @param info - дополнительные данные, на каждый ключ энума
     *  содержит объект, в том числе содержащий имя в поле name
     */
    function EnumHelpers(values, info) {
        this.values = values;
        this.info = info;
    }
    /**
     * Возвращает массив ключей энума
     */
    EnumHelpers.prototype.keys = function () {
        return Object.values(this.values);
    };
    /**
     * Возвращает массив опций энума.
     * Подходит для селектов, табов и т.п.
     *
     * @field - поле, которое возвращается в качестве label
     * @labelKey - ключ, в который записывается label
     * @valueKey - ключ, в который записывается ключ энума
     */
    EnumHelpers.prototype.getOptions = function (field, labelKey, valueKey) {
        var _this = this;
        if (field === void 0) { field = 'name'; }
        if (labelKey === void 0) { labelKey = 'label'; }
        if (valueKey === void 0) { valueKey = 'value'; }
        return Object.keys(this.values).map(function (key) {
            var _a;
            var info = _this.info[key];
            return tslib_1.__assign(tslib_1.__assign({}, info), (_a = {}, _a[labelKey] = info[field], _a[valueKey] = _this.values[key], _a));
        });
    };
    /**
     * Возвращает имя по ключу
     * @param value - ключ энума
     */
    EnumHelpers.prototype.getName = function (value) {
        var _a;
        return (_a = this.info[value]) === null || _a === void 0 ? void 0 : _a.name;
    };
    /**
     * Возвращает ключ по имени
     * @param name - имя
     */
    EnumHelpers.prototype.getByName = function (name) {
        for (var _i = 0, _a = Object.keys(this.info); _i < _a.length; _i++) {
            var key = _a[_i];
            if (this.info[key].name === name) {
                return key;
            }
        }
        return undefined;
    };
    /**
     * Возвращает ключ по произвольному значению
     * @param value - имя
     * @param field - поле, с которым сравнивается значение
     */
    EnumHelpers.prototype.getBy = function (value, field) {
        for (var _i = 0, _a = Object.keys(this.info); _i < _a.length; _i++) {
            var key = _a[_i];
            if (this.info[key][field] === value) {
                return key;
            }
        }
        return undefined;
    };
    return EnumHelpers;
}());
exports.EnumHelpers = EnumHelpers;
//# sourceMappingURL=EnumHelpers.js.map